local L = LibStub("AceLocale-3.0"):GetLocale("NeedjaTrackerPlus", true)

NTP = LibStub("AceAddon-3.0"):NewAddon("NeedjaTrackerPlus", "AceConsole-3.0", "AceEvent-3.0")

local S2K = LibStub("S2Kmisc-1.0")

function NTP:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("NeedjaTrackerPlusDB", self:Defaults(), true)

    self.db.RegisterCallback(self, "OnProfileReset", "ProcessConfigUpdate")
    self.db.RegisterCallback(self, "OnProfileChanged", "ProcessConfigUpdate")
    self.db.RegisterCallback(self, "OnProfileCopied", "ProcessConfigUpdate")

    LibStub("AceConfig-3.0"):RegisterOptionsTable("NeedjaTrackerPlus", self:Options())

    local dlg = LibStub("AceConfigDialog-3.0")

    dlg:AddToBlizOptions("NeedjaTrackerPlus", L.title, nil, "general")
    dlg:AddToBlizOptions("NeedjaTrackerPlus", L.cfg_profiles, L.title, "profiles")

    ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", function(this, ...) return self:FilterChatMessage(...) end)

    self:ProcessConfigUpdate()
end

function NTP:OnEnable()
    self:RegisterEvent("CHAT_MSG_LOOT", "ProcessChatMessage")
end

function NTP:OnDisable()
end

function NTP:FilterChatMessage(event, msg)
    return self:GetProfileValue("enable") and S2K:ParseMsgMatchTable(S2K.LOOT_ROLL_MSG_MATCH_TABLE, msg)
end

function NTP:ProcessChatMessage(event, msg)
    local handler, args = S2K:ParseMsgMatchTable(S2K.LOOT_ROLL_MSG_MATCH_TABLE, msg)

    if handler == "LOOT_ROLL_CHOICE" then
        self:ProcessRollChoice(args)

    elseif handler == "LOOT_ROLL_ROLLED" then
        self:ProcessRollRolled(args)

    elseif handler == "LOOT_ROLL_ALL_PASSED" then
        self:ProcessRollAllPassed(args)

    elseif handler == "LOOT_ROLL_WON" then
        self:ProcessRollWon(args)

    elseif handler == "LOOT_ROLL_WON_NO_SPAM" then
        self:ProcessRollWonNoSpam(args)
    end
end

function NTP:ProcessConfigUpdate()
    if self:GetProfileValue("enable") then
        self:Enable()
    else
        self:Disable()
    end
end

function NTP:GetProfileValue(key)
    return self.db.profile[key]
end

function NTP:SetProfileValue(key, value)
    self.db.profile[key] = value
end

function NTP:GetCharValue(key)
    return self.db.char[key]
end

function NTP:SetCharValue(key, value)
    self.db.char[key] = value
end

function NTP:__GetRollItemData(item)
    local ritem = self:GetCharValue("current")[item]
    if not ritem then
        ritem = { players = {}, need = 0, greed = 0, disenchant = 0, unknown = 0 }
        self:GetCharValue("current")[item] = ritem
    end
    return ritem
end

function NTP:__GetAndRemoveRollItemData(item)
    local ritem = self:GetCharValue("current")[item]
    if not ritem then
        ritem = { item = item, players = {}, need = 0, greed = 0, disenchant = 0, unknown = 0 }
    else
        self:GetCharValue("current")[item] = nil
    end
    return ritem
end

function NTP:__GetRollPlayerData(ritem, player)
    local rplayer = ritem.players[player]
    if not rplayer then
        rplayer = { player = player }
        ritem.players[player] = rplayer
    end
    return rplayer
end

function NTP__RecalcChoiceCounters(ritem)
    ritem.need = 0 ritem.greed = 0 ritem.disenchant = 0 ritem.unknown = 0
    local rplayer
    for _, rplayer in pairs(ritem.players) do
        local choice = rplayer.choice or "unknown"
        ritem[choice] = ritem[choice] + 1
    end
end

function NTP:ProcessRollChoice(args)
    local ritem = self:__GetRollItemData(args.item)
    local rplayer = self:__GetRollPlayerData(ritem, args.player)

    player.choice = args.choice

    self:__RecalcChoiceCounters()

    -- report roll choice
end

function NTP:ProcessRollRolled(args)
    local ritem = self:__GetRollItemData(args.item)
    local rplayer = self:__GetRollPlayerData(ritem, args.player)

    player.choice = args.choice
    player.roll = args.roll

    self:__RecalcChoiceCounters()
end

function NTP:ProcessRollAllPassed(args)
    local ritem = self:__GetAndRemoveRollItemData(args.item)

    -- add to statistics
    -- add to history
    -- report roll result
end

function NTP:ProcessRollWon(args)
    local ritem = self:__GetAndRemoveRollItemData(args.item)
    local rplayer = self:__GetRollPlayerData(ritem, args.player)

    ritem.won = args.player

    -- add to statistics
    -- add to history
    -- report roll result
end

function NTP:ProcessRollNoSpam(args)
    local ritem = self:__GetAndRemoveRollItemData(args.item)
    local rplayer = self:__GetRollPlayerData(ritem, args.player)

    ritem.won = args.player

    player.choice = args.choice
    player.roll = args.roll

    self:__RecalcChoiceCounters()

    -- add to statistics
    -- add to history
    -- report roll result
end
